/* problems.c - By Luke Sheneman.  1992 */

/* Modified: 2/14/93, DC.
 * Added FULLRECORD/PARTIALRECORD field to all view_reopen() and
 * get_problem() calls.
 */

/*
 * Copyright (c) 1992 Luke Sheneman
 * Copyright (c) 1992 University of Idaho, Moscow, Idaho.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation free of charge for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the University of Idaho
 * not be used in advertising or publicity pertaining to distribution of
 * the software without specific, written prior permission.  The University
 * of Idaho makes no representations about the suitability of this software
 * for any purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE UNIVERSITY OF IDAHO DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE UNIVERSITY OF IDAHO BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */


/*
 * 
 *	problem.c  - this source file includes the following functions:
 *
 *	FormatProblemTypes() - Returns an array of strings containing the 
 *		paths which can be followed when traversing the problem tree
 *		at that point in the tree.
 *      ReadProblem() - Returns a single string with embedded new-lines which 
 *		contains an entire problem record formatted suitable for 
 *		displaying or printing.
 *	LockProblem() and UnlockProblem() - Used to insure that only one person
 * 		is editing a problem at a time.  Lock needs to return success
 *		or failure if it can/can't lock a file (after a couple of 
 * 		seconds).
 *	SolveProblem() - Marks a problem as solved.
 *	ReopenProblem() - Records a request by a user to reopen a solved 
 * 		problem.
 *	ReopenRequest() - Marks a problem as being requested by a user for 
 *		reopening.
 *	DenyReopen() - Sends a "reason text" message to the user who requested
 * 		a problem be reopened.
 * 	ReportAgain() - Allows a user to add themselves to the list of reporters
 *		for an unsolved problem.
 *	DeleteProblem() - Removes a problem from the database.
 *	MoveProblem() - Moves a problem from one location to another in the 
 * 		problem tree.
 *	LinkProblem() - Makes a problem appear to be in two places in the 
 * 		problem tree at once.
 *	PrintProblem() - Print a problem.
 * 	WriteProblem() - Creates a new problem entry in the problem database.
 *	AppendProblem() - Appends a string to an existing problem log.
 * 	IsSysop() - determines if the effective group id of the user places
 *		that user in the SYSOPGID (the group of sysops)
 *	CopyProblemRecord() - Copies a problem record recieved from the 
 *		zdbm functions into a newly allocated problem record.
 *	FixTime() - Removes the offending newline from calling ctime().
 *	IsReopened() - Calls a zdbm function to see if a desired problem is 
 *   		requested to be reopened or not.
 */

#include <sys/types.h>
#include <stdio.h>
#include "zdbm.h"
#include "cloud.h"
#include "clouderror.h"


int FormatProblemTypes(tree,path,problem,control,types)
ProblemTree *tree;
char *path;
int problem;
int control;
char *types[];
{
	int loop;
	char newpath[DBPATHLEN];

	cld_errno=0;

#ifdef DEBUG
	printf("Path before fixing in FormatProblemTypes() = %s\n",path);
#endif /* DEBUG */

	newpath[0]='\0';
	strcpy(newpath,FixPath(path));
	if(!strcmp(newpath,""))
		sprintf(path,"ROOT");
	else
		sprintf(path,"ROOT/%s",newpath);

#ifdef DEBUG
	printf("Path after fixing in FormatProblemTypes() = %s\n",path);
#endif /* DEBUG */

#ifdef DEBUG
	printf("PATH IN FORMATPROBLEMTYPES() = ***%s***\n",path);
#endif /* DEBUG */

        switch(control)
        {
                case ABS_TYPE:
		{	
			ProblemTree *subhead,*t;
#ifdef DEBUG
			printf("Just before call to SubTree() tree=%p path=%s pathptr=%p\n",tree,path,
				path);
#endif /* DEBUG */
			t=SubTree(tree,path);
#ifdef DEBUG
			printf("Just AFTER call to SubTree()\n");
#endif /* DEBUG */
			if(!t)
			{
				cld_errno=CLD_ERR10;
				return(-1);
			}
			else
			{

				subhead=t->subhead;
				if(!subhead)
					return(NULL);

				loop=0;
				while( (subhead) && (loop<MAXBRANCHES) )
				{
					strcpy(types[loop],subhead->path);
					subhead=subhead->next;	
#ifdef DEBUG
					printf("-->%s\n",types[loop]) ;
#endif
					loop++;
				}
				types[loop][0]='\0';
			}	
                        break;
		}
                case INC_TYPE:
		{
			ProblemTree *subhead,*t;	
			int pos;

			t=SubTree(tree,path);
			if(!t)
			{
				cld_errno=CLD_ERR10;
				return(-1);
			}
			else
			{
				subhead=t->subhead;
				if(!subhead)
				{
					cld_errno=CLD_ERR11;
					return(-1);
				}
				pos=0;
				loop=0;
				while( (subhead) && (pos!=problem) )
				{
					subhead=subhead->next;
					pos++;
				}
				if(!subhead)	/* desired subhead not found */
				{
					cld_errno=CLD_ERR12;
					return(-1);			
				}

				strcpy(path,ROOTSTR);
				strcat(path,subhead->fullpath);
				if(pos<problem)
				{
					cld_errno=CLD_ERR13;
					return(-1);
				}
				else
				{
					subhead=subhead->subhead;
					if(!subhead)
						return(NULL);	/* NO CHILDREN */
					while( (subhead) && (loop<MAXBRANCHES) )
					{
						strcpy(types[loop],subhead->path);
						subhead=subhead->next;
						loop++;
					}
					types[loop][0]='\0';
				}
			}

                        break;
		}
                case PREV_TYPE:
		{
			ProblemTree *subhead,*t;
			char *parts[MAXBRANCHES];
			char parent[DBPATHLEN];
			int loop,end;


			BreakUpPath(path,parts);	
			end=0;
			while(strcmp(parts[end++],""));
			parent[0]='\0';
			for(loop=0;loop<end-2;loop++)
				sprintf(parent,"%s/%s",parent,parts[loop]);	
			if(!strcmp(parent,"/ROOT"))			/*  KLUDGE-O-RAMA  */
				strcpy(parent,ROOTSTR);
			strcpy(path,parent);
			if(!strcmp(path,""))
			{
				FreeParts(parts);
				return(NULL);	/* no previous types - at ROOT */
			}

			t=SubTree(tree,path);
			if(!t)
			{
				cld_errno=CLD_ERR10;
				FreeParts(parts);
				return(-1);
			}
			else
			{
				subhead=t->subhead;
				if(!subhead)
				{
					FreeParts(parts);
					return(NULL);	/* NO CHILDREN */
				}
				loop=0;
				while( (subhead) && (loop<MAXBRANCHES) )
				{
					strcpy(types[loop],subhead->path);
					subhead=subhead->next;	
					loop++;
				}
				types[loop][0]='\0';
			}	
			
                        break;
		}
                default:
		{
			cld_errno=CLD_ERR13;	
			return(-1);
		}
        }
	return(True);
}



problem_record *ReadProblem(tree,path,problem,summarylist)
ProblemTree *tree;
char *path;
int problem;
Summary *summarylist;
{
	Summary *ts;
	int counter;
	problem_record *res;
	static problem_record *realres=NULL;
	char newpath[DBPATHLEN];
	int loop;

	cld_errno=0;

#ifdef DEBUG
	printf("In ReadProblem.  path=%s   problem#=%d\n",path,problem);
#endif /* DEBUG */

	strcpy(newpath,FixPath(path));

#ifdef DEBUG
	ts=summarylist;
	printf("******  Traversing summarylist.\n");
	while(ts)
	{
		printf("SUMMARY: PRID=%s\n",ts->prid);
		printf("	 PATH=%s\n",ts->path);
		printf("      SUMMARY=%s\n",ts->summary);
		printf("      STATUS=%d\n",ts->status);
		ts=ts->next;
	}
#endif /* DEBUG */

	counter=0;
	ts=summarylist;
	while( (ts) && (counter<problem) )
	{
#ifdef DEBUG
		printf("In while() loop. Traversing summarylist.  ts->summary=%s\n",ts->summary); 
#endif /* DEBUG */
		counter++;
		ts=ts->next;
	}
	if(!ts)
	{
		cld_errno=CLD_ERR14;	
		return(NULL);	/* ERROR! */
	}

	res=get_problem(newpath,ts->prid,PARTICULAR,NULL,LINKS,FULLRECORD);/*Modified: 2/14/93, DC*/
	if(!res)
	{
		cld_errno=CLD_ERR15;	
		return(NULL);	
	}
	else
	{
		if(realres)
		{
#ifdef DEBUG
			printf("In ReadProblem() - About to free a problem record.\n");
#endif /* DEBUG */
			FreeProblemRecord(realres);
		}
		realres=(problem_record *)malloc(sizeof(problem_record));
		CopyProblemRecord(realres,res);
		return(realres);
	}
}



int LockProblem(path,problem)
char *path;
problem_record *problem;
{
	int res;
	char newpath[DBPATHLEN];

	cld_errno=0;

#ifdef DEBUG
	printf("The path at beginning of LockProblem() = %s\n",path);
#endif /* DEBUG */

	strcpy(newpath,FixPath(path));

#ifdef DEBUG
	printf(" In LockProblem() - newpath = %s\n",newpath);
#endif /* DEBUG */

	res=edit_log(newpath,problem->prid,NULL,START);
	if(!res)
	{
		cld_errno=CLD_ERR16;
		return(NULL);
	}
	else
		return(True);
}




int UnlockProblem(path,problem)
char *path;
problem_record *problem;
{
	int res;
	char newpath[DBPATHLEN];

	cld_errno=0;

	strcpy(newpath,FixPath(path));

#ifdef DEBUG
	printf(" In UnlockProblem() - newpath = %s\n",newpath);
#endif /* DEBUG */

	res=edit_log(newpath,problem->prid,NULL,END);
	if(!res)
	{
		cld_errno=CLD_ERR17;
		return(NULL);
	}
	else
		return(True);
}





int SolveProblem(tree,path,problem,solution,userinfo)
ProblemTree *tree;
char *path;
problem_record *problem;
char *solution;
UserInfo *userinfo;
{
	int res,loop;
	char newpath[DBPATHLEN];
	char sys[100];
/*	char filename[FNAMELEN];*/
	char filename[COMPLETEPATHLEN];
	char reporter[MAXNAME];
	FILE *fp;

	cld_errno=0;

#ifdef DEBUG
	printf("At very beginning of SolveProblem()\n");
#endif /* DEBUG */

	strcpy(newpath,FixPath(path));

#ifdef DEBUG
	printf("In SolveProblem() - newpath = %s\n",newpath);
#endif /* DEBUG */

	res=AppendProblem(tree,path,problem,userinfo,solution);
#ifdef DEBUG
	printf("In SolveProblem(), just returned from AppendProblem()(res=%d)\n",res) ;
#endif
	res=solve_problem(newpath,problem->prid);
	if(!res)  
	{
		cld_errno=CLD_ERR18;
		return(NULL);
	}
	else
	{
#ifdef DEBUG
	printf("In SolveProblem(), just returned from solve_problem()\n") ;
#endif
		for (loop=0; loop < problem->num_rep; loop++)
		{
			strcpy(filename,(char *)mktemp(TEMPLATE));

			fp=fopen(filename,"w");
			if(!fp)
			{	
				cld_errno=CLD_ERR21;	/* fix this */
				return(NULL);
			}
			else
			{
				strcpy(reporter,((problem->reporter_list)[loop]).account_name);
				fprintf(fp,"Dear %s,\n\n",reporter);
				fprintf(fp,"	A problem in the PTS database has been solved in which you were\n");
				fprintf(fp,"listed as a reporter.  You are hereby being notified that this problem has\n");	
				fprintf(fp,"been solved by the support staff.\n\n");
				fprintf(fp,"The problem log for this problem is as follows:\n\n");
				fprintf(fp,"_____________________________________________________________________\n");
				fprintf(fp,"%s\n\n",problem->log_file);
				fprintf(fp,"_____________________________________________________________________\n");
				fprintf(fp,"Thank you for your support.\n\n");
				fclose(fp);
				sprintf(sys,"%s %s < %s",SOLVED_MAILER,reporter,filename);
#ifdef DEBUG
				printf("SolveProblem(): \nreporter=%s\nsys='%s'\n",reporter,sys) ;
#endif
				system(sys);
				unlink(filename);
			}
		}
		return(True);
	}
}



problem_record *ReopenProblem(tree,path,problem,sysop_string)
ProblemTree *tree;
char *path;
problem_record *problem;
char *sysop_string;
{
	char *prid;	
	problem_record *res;
	static problem_record *realres=NULL;

	cld_errno=0;
	
	prid=confirm_reopen(((reopened_record *)(problem->reop_rec))->prid,
		&(((reopened_record *)(problem->reop_rec))->
		rep_rec),sysop_string,((reopened_record *)(problem->reop_rec))->
		reasons_log);

	if(!prid)
	{
		cld_errno=CLD_ERR19;
		return(NULL);
	}
	else
	{
		res=get_problem( ((reopened_record *)(problem->reop_rec))->db_path,prid,PARTICULAR,
			NULL,LINKS,FULLRECORD);/*Modified: 2/14/93, DC*/
		if(!res)
		{
			cld_errno=CLD_ERR35;
			return(NULL);
		}
		else
		{
			if(realres)
				FreeProblemRecord(realres);
			realres=(problem_record *)malloc(sizeof(problem_record));
			CopyProblemRecord(realres,res);
			return(realres);
		}
	}
}



int ReopenRequest(path,problem,reason,user)
char *path;
problem_record *problem;
char *reason;
UserInfo *user;
{
	int res,t;
	reporter_record report;
	char hostname[MAXHOSTNAMELEN];
	char newpath[DBPATHLEN];

	cld_errno=0;

	strcpy(newpath,FixPath(path));

#ifdef DEBUG 
	printf("In ReopenRequest: newpath=%s\n",newpath);
#endif /* DEBUG */

	gethostname(hostname,MAXHOSTNAMELEN);
	strcpy(report.node_name,hostname);
	strcpy(report.account_name,user->username);
	t=time(0);
	strcpy(report.date_time,FixTime((char *)ctime((time_t *)&t)));

	res=reopen_request(newpath,problem->prid,reason,&report);
	if(!res)
	{
		cld_errno=CLD_ERR20;
		return(NULL);
	}
	else
		return(True);
}





int DenyReopen(tree,path,problem,user,deny)
ProblemTree *tree;
char *path;
problem_record *problem;
UserInfo *user;
char *deny;
{
	char username[MAXNAME];
	char realname[MAXNAME];
	char reporter[MAXNAME];
	char filename[FNAMELEN];
	char node[MAXHOSTNAMELEN];
	char sys[100];
	FILE *fp;
	int res;
	
	cld_errno=0;

	strcpy(username,user->username);
	strcpy(realname,user->realname);
	strcpy(node,((reopened_record *)(problem->reop_rec))->rep_rec.node_name);
	strcpy(reporter,((reopened_record *)(problem->reop_rec))->rep_rec.account_name);
	strcpy(filename,(char *)mktemp(TEMPLATE));

	fp=fopen(filename,"w");
	if(!fp)
	{
		cld_errno=CLD_ERR21;	
		return(NULL);
	}
	else
	{
		fprintf(fp,"Dear %s,\n\n",reporter);
		fprintf(fp,"      Your request to reopen a problem in the XPTS database was denied.\n\n");
		fprintf(fp,"Request Denied By: %s  (%s)\n",username,realname);
		fprintf(fp,"Reason For Denial:\n\n\n");
		fprintf(fp,"%s",deny);
		fflush(fp);
		sprintf(sys,"%s %s@%s < %s",MAILER,reporter,node,filename);
		system(sys);
		
		fclose(fp);
		unlink(filename);	
		
		res=deny_reopen(((reopened_record *)(problem->reop_rec))->prid);
		if(!res)
		{
			cld_errno=CLD_ERR22;
			return(NULL);
		}
		else
			return(True);
	}
}





problem_record *ReportAgain(path,userinfo,problem)
char *path;
UserInfo *userinfo;
problem_record *problem;
{
	int res;
	reporter_record report;
	static problem_record *realres=NULL;
	problem_record *problem_rec;
        char hostname[MAXHOSTNAMELEN];
	char newpath[DBPATHLEN];
	int t;

	cld_errno=0;
	
	strcpy(newpath,FixPath(path));

#ifdef DEBUG
	printf("ReportAgain() - newpath = %s\n",newpath);
#endif /* DEBUG */

        gethostname(hostname,MAXHOSTNAMELEN);
        strcpy(report.node_name,hostname);
        strcpy(report.account_name,userinfo->username);
        t=time(0);
        strcpy(report.date_time,FixTime((char *)ctime((time_t *)&t)));

	res=(int)add_reporter(newpath,problem->prid,&report);
	if(!res)
	{
		cld_errno=CLD_ERR23;		
		return(NULL);
	}
	else
	{
		problem_rec=get_problem(newpath,problem->prid,PARTICULAR,NULL,LINKS,FULLRECORD);/*Modified: 2/14/93, DC*/
		if(!problem_rec)
		{
			cld_errno=CLD_ERR23;
			return(NULL);
		}
		else
		{
			if(realres)
				FreeProblemRecord(realres);	
			realres=(problem_record *)malloc(sizeof(problem_record));
			CopyProblemRecord(realres,problem_rec);
			return(realres);
		}
	}
}

		

int DeleteProblem(tree,path,problem)
ProblemTree *tree;
char *path;
problem_record *problem;
{
	int res;
	char newpath[DBPATHLEN];

	cld_errno=0;

	strcpy(newpath,FixPath(path));

#ifdef DEBUG
	printf("DeleteProblem() newpath = %s\n",newpath);
#endif /* DEBUG */

	res=delete_problem(newpath,problem->prid,DELETEOK);
	if(!res)
	{
		cld_errno=CLD_ERR24;
		return(NULL);
	}
	else
		return(True);
}



int MoveProblem(tree,path1,problem,path2)
ProblemTree *tree;
char *path1;
problem_record *problem;
char *path2;
{
	int res;
	char newpath1[DBPATHLEN],newpath2[DBPATHLEN];
	char temp_path[DBPATHLEN];
	ProblemTree *temp_sub_tree;

	cld_errno=0;

	strcpy(newpath1,FixPath(path1));/* Make Sure Paths Are Usable. */
	strcpy(newpath2,FixPath(path2));


	sprintf(temp_path,"ROOT/%s",newpath1);	/* Check To See If Paths Are Valid. */
	temp_sub_tree=SubTree(tree,temp_path);
	if( (!temp_sub_tree) || (temp_sub_tree->subhead) )
	{
		cld_errno=CLD_ERR25;		
		return(NULL);
	}

	sprintf(temp_path,"ROOT/%s",newpath2);
	temp_sub_tree=SubTree(tree,temp_path);
	if( (!temp_sub_tree) || (temp_sub_tree->subhead) )
	{
		cld_errno=CLD_ERR26;
		return(NULL);
	}


#ifdef DEBUG
	printf("This is in MoveProblem() - path1=%s  path2=%s\n",newpath1,newpath2);
#endif /* DEBUG */

	res=move_problem(newpath1,newpath2,problem->prid);
	if(!res)	
	{
		cld_errno=CLD_ERR27;
		return(NULL);
	}
	else
		return(True);
}



int LinkProblem(tree,path1,problem,path2)
ProblemTree *tree;
char *path1;
problem_record *problem;
char *path2;
{
	int res;
	char newpath1[DBPATHLEN],newpath2[DBPATHLEN];
	char temp_path[DBPATHLEN];
	ProblemTree *temp_sub_tree;

	cld_errno=0;

	strcpy(newpath1,FixPath(path1));
	strcpy(newpath2,FixPath(path2));


	sprintf(temp_path,"ROOT/%s",newpath1);	/* Check To See If Paths Are Valid. */
	temp_sub_tree=SubTree(tree,temp_path);
	if( (!temp_sub_tree) || (temp_sub_tree->subhead) )
	{
		cld_errno=CLD_ERR28;
		return(NULL);
	}

	sprintf(temp_path,"ROOT/%s",newpath2);
	temp_sub_tree=SubTree(tree,temp_path);
	if( (!temp_sub_tree) || (temp_sub_tree->subhead) )
	{
		cld_errno=CLD_ERR29;
		return(NULL);
	}

#ifdef DEBUG
	printf("This is in LinkProblem() - path1=%s  path2=%s\n",newpath1,newpath2);
#endif /* DEBUG */

	res=link_problem(newpath1,newpath2,problem->prid);
	if(!res)
	{
		cld_errno=CLD_ERR30;
		return(NULL);
	}
	else
		return(True);
}




int PrintProblem(tree,path,problem)
ProblemTree *tree;
char *path;
problem_record *problem;
{
	char filename[FNAMELEN];
	char sys[100];
	FILE *fp;
	reporter_record *reporter;
	int i,j,buflen;
	char newpath[DBPATHLEN];
	int maxname=MAXNAMELEN-8;

	cld_errno=0;

	strcpy(newpath,FixPath(path));

	strcpy(filename,(char *)mktemp(TEMPLATE));
	fp=fopen(filename,"w");
	if(!fp)
	{
		cld_errno=CLD_ERR31;
		return(False);
	}
	else
	{
		fprintf(fp,"PROBLEM TYPE:\t%s\n",newpath);
		fprintf(fp,"STATUS:\t\t");
		switch(problem->status)
		{
			case UNSOLVED:
				fprintf(fp,"Unsolved\n\n");
				break;
			case SOLVED:
				fprintf(fp,"Solved\n\n");
				break;
			case REOPENED:
				fprintf(fp,"Reopen Request\n\n");
				break;
			default:
				break;
		}
		
		fprintf(fp,"SUMMARY:\n");
		fprintf(fp,"%s\n\n",problem->short_description);
		
		fprintf(fp,"%-*s%-*s%s\n",maxname,"REPORTER",FNAMELEN,"NODE","DATE/TIME");
		fprintf(fp,"%-*s%-*s%s\n",maxname,"--------",FNAMELEN,"----","---------");


		reporter=problem->reporter_list;
		if(reporter)
		{
			for(i=0;i<problem->num_rep;++i)
			{
				fprintf(fp,"%s",(reporter[i]).account_name);
				buflen=strlen((reporter[i]).account_name);
				for(j=buflen;j<maxname;++j)
					fprintf(fp," ");

				fprintf(fp,"%s",(reporter[i]).node_name);
				buflen=strlen((reporter[i]).node_name);
				for(j=buflen;j<FNAMELEN;++j)
					fprintf(fp," ");

				fprintf(fp,"%s",(reporter[i]).date_time);	

				fprintf(fp,"\n");
			}
		}
		fprintf(fp,"\n");

		fprintf(fp,"PROBLEM DESCRIPTION:\n");
		fprintf(fp,"%s",problem->log_file);
		fprintf(fp,"\n"); 
					

		fclose(fp);
		sprintf(sys,"%s %s %s",PRINTER,PRINTER_OPTS,filename);
		system(sys);
		unlink(filename);
		return(True);
	}
}



int WriteProblem(tree,path,userinfo,description,summary,node)
ProblemTree *tree;
char *path;
UserInfo *userinfo;
char *description;
char *summary;
char *node;
{
	int res,t;
        reporter_record report;
	char newpath[DBPATHLEN];

	cld_errno=0;

	strcpy(newpath,FixPath(path));

#ifdef DEBUG
	printf("In WriteProblem().  newpath=%s\n",newpath);
#endif /* DEBUG */

        strcpy(report.node_name,node);
        strcpy(report.account_name,userinfo->username);
        t=time(0);
        strcpy(report.date_time,FixTime((char *)ctime((time_t *)&t)));


#ifdef DEBUG
	printf("path=%s in WriteProblem() after FixPath() call\n",path);
	printf("newpath=%s in WriteProblem() after FixPath() call\n",newpath);
#endif /* DEBUG */

	res=(int)add_problem(&report,description,summary,newpath);
	if(!res)
	{
		cld_errno=CLD_ERR32;	
		return(NULL);
	}
	else
		return(True);
}






int AppendProblem(tree,path,problem,userinfo,text)
ProblemTree *tree;
char *path;
problem_record *problem;
UserInfo *userinfo;
char *text;
{
	int res;
	char newpath[DBPATHLEN];

	cld_errno=0;

	strcpy(newpath,FixPath(path));

#ifdef DEBUG
	printf("AppendProblem() - newpath = %s\n",newpath);
#endif /* DEBUG */

	res=edit_log(newpath,problem->prid,text,END);

#ifdef DEBUG
	printf("AppendProblem() - res = %d\n",res);
#endif /* DEBUG */

	if(!res)
	{
		cld_errno=CLD_ERR33;	
		return(NULL);
	}
	else
		return(True);

}






/* IsSysop looks at the UserInfo structure that was passed to it from the ui.
   It must be called with a UserInfo structure that has already been 
   initialized by calling Setup().  IsSysop returns a positive integer if 
   the user's effective group id (gid) is currently the same as the sysop's, 
   as defined in the constant SYSOPGID.  If the user's effective group id
   is NOT the SYSOPGID, then a zero (0) is passed back.

   Luke Sheneman.  	Wed Jul 22 12:02:22 PDT 1992
*/

int IsSysop(userinfo)
UserInfo *userinfo;
{
	cld_errno=0;
	return( userinfo->sysopflag );
}





/* Expects realres to be allocated, and res to be allocated and filled  */

void CopyProblemRecord(realres,res)
problem_record *realres;
problem_record *res;
{
	int loop;

	cld_errno=0;

	strcpy(realres->prid,res->prid);
	strcpy(realres->short_description,res->short_description);
	realres->num_rep=res->num_rep;

	realres->reporter_list=(reporter_record *)malloc(sizeof(reporter_record)*realres->num_rep);
	for(loop=0;loop<realres->num_rep;loop++)
	{
		strcpy(realres->reporter_list[loop].account_name,res->reporter_list[loop].account_name);
		strcpy(realres->reporter_list[loop].node_name,res->reporter_list[loop].node_name);
		strcpy(realres->reporter_list[loop].date_time,res->reporter_list[loop].date_time);
	}

	realres->num_old_rep=res->num_old_rep;
	realres->old_reporter_list=(reporter_record *)malloc(sizeof(reporter_record)*realres->num_old_rep);
	for(loop=0;loop<realres->num_old_rep;loop++)
	{
		strcpy(realres->old_reporter_list[loop].account_name,res->old_reporter_list[loop].account_name);
		strcpy(realres->old_reporter_list[loop].node_name,res->old_reporter_list[loop].node_name);
		strcpy(realres->old_reporter_list[loop].date_time,res->old_reporter_list[loop].date_time);
	}

	realres->log_file=(char *)malloc(sizeof(char)*strlen(res->log_file));
	strcpy(realres->log_file,res->log_file);

	realres->status=res->status;
}


char *FixTime(time)
char *time;
{
	cld_errno=0;

	time[strlen(time)-1]='\0';
	return(time);
}



int IsReopened(path,problem)
char *path;
problem_record *problem;
{
	cld_errno=0;
	return(is_reopened(path,problem->prid));
}


void FreeProblemRecord(rec)
problem_record *rec;
{
	int loop;

	if(!rec)
		return;

#ifdef DEBUG
	printf("In FreeProblemRecord()\n");
#endif /* DEBUG */

/* This needs to work in order to free this properly.  */

/*
	for(loop=0;loop<(rec->num_rep);loop++)
		if( (rec->reporter_list)[loop] )
			free( (rec->reporter_list)[loop] );
*/
#ifdef DEBUG
	printf("In FreeProblemRecord().  Just freed reporter's list.\n");
#endif /* DEBUG */

/*
	for(loop=0;loop<(rec->num_old_rep);loop++)
		if(((rec->old_reporter_list)[loop])!=NULL)
			free( (rec->old_reporter_list)[loop] );
*/

#ifdef DEBUG
	printf("In FreeProblemRecord().  Just freed old reporter's list.\n");
#endif /* DEBUG */

	if(rec)
		free(rec);

#ifdef DEBUG
	printf("In FreeProblemRecord().  Just freed the reporter record.\n");
#endif /* DEBUG */

}
