/*
 * get_problem.c -- 
 * Chris Hyde
 * created: 
 */

/*
 * Copyright (c) 1992 Christopher G. Hyde
 * Copyright (c) 1992 University of Idaho, Moscow, Idaho.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation free of charge for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the University of Idaho
 * not be used in advertising or publicity pertaining to distribution of
 * the software without specific, written prior permission.  The University
 * of Idaho makes no representations about the suitability of this software
 * for any purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE UNIVERSITY OF IDAHO DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE UNIVERSITY OF IDAHO BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

#include "zdbm.h"


problem_record *get_problem(char *db_path, char *prid, int type,
                            int direction, int links, int whole_record)
{
  static problem_record pro_rec;
  problem_record *errptr; /* used to hold return value from create_record() */
  static char current_db[DBPATHLEN];
  static int old_type=0;
  index_record index, *errpointer;
  int new=FALSE;
  FILE *db_pointer;

#ifdef DEBUG
 printf("\nIn get_problem()\n");
 printf("\tLeaf to search - %s\n", db_path);
 printf("\tPrid to find - %s\n", prid);
 printf("\tType of problem - %d\n\tDirecrtion of search - %d\n",
		type, direction);
 printf("\tFollow links = %d\n", links);
 printf("\tWhole record = %d\n", whole_record);
#endif

 db_errorno=0;

/* We want to start a new search if either the leaf to search changes, or the
 * type of search changes.
 */

  if (strcmp(current_db, db_path)!=0)
  {	/* searching new leaf */
#ifdef DEBUG
 printf("get_problem() - searching new leaf\n");
#endif
     nl_strncpy(current_db, db_path, DBPATHLEN);
     new=TRUE;
  }
#ifdef DEBUG
 else
    printf("get_problem() - searching previous leaf\n");
#endif

  if (old_type!=type)
  {	/* searching for new type */
#ifdef DEBUG
 printf("get_problem() - searching for new type\n");
#endif
     old_type=type;
     new=TRUE;
  }
#ifdef DEBUG
 else
    printf("get_problem() - searching for previous type\n");
#endif

  if ((db_pointer=open_file(db_path, ".index", "r+", INDEX))==NULL)
     return(NULL);  /* couldn't open the new database */

  switch (type) /* get the problem wanted */
  {
     case SOLVED:        errpointer=next_solved(db_pointer, new, links,
						 &index);
			 break;

     case UNSOLVED:      errpointer=next_unsolved(db_pointer, new, links,
						   &index);
			 break;

     case PARTICULAR:    errpointer=find_problem(db_pointer, prid, &index);
			 break;

     default:            db_errorno=BADTYPE;
			 errpointer=NULL;
  } /* end switch */

  if (errpointer==NULL) /* didn't find the problem or error */
  {
     close_file(db_pointer);
     return(NULL);
  }
  else
     close_file(db_pointer);
#ifdef DEBUG
 printf("get_problem() - found index entry of desired problem\n");
#endif

  if (whole_record==FULLRECORD)
  {
#ifdef DEBUG
 printf("get_problem() - caller wants full problem record\n");
#endif

     errptr=create_record(db_path, &index, &pro_rec);
  } /* end caller wants full record */
  else /* the caller only wants part of the record */
  {
#ifdef DEBUG
 printf("get_problem() - caller wants a partial problem record\n");
#endif

     errptr=partial_record(db_path, &index, &pro_rec);
  } /* end else partial record */

  if (errptr==NULL)
     return(NULL); /* problem creating the problem record */
  else
  {
#ifdef DEBUG
 printf("get_problem() - created problem record\n");
 printf("\tProblem gotten is:\n\n");
 print_prob(&pro_rec);
#endif

  return(&pro_rec);
  } /* end created problem_record */
} /* end get_problem */


index_record *next_solved(FILE *db_pointer, int new, int links,
			  index_record *index)
{
  static long start;
  long current;
  int no_solved;

#ifdef DEBUG
 printf("\nIn next_solved() - searching ");
 if (new)
    printf("NEW leaf\n");
 else
    printf("PREVIOUS leaf\n");
#endif

  if (new) /* searching a new database leaf */
     current=start=0;
  else /* searching old leaf so seek to last position */
  {
     fseek(db_pointer, start, SEEK_SET);
     current=start;
  }
  no_solved=TRUE;

  while (no_solved)
  {
     if (fread(index, sizeof(index_record), 1, db_pointer)==NULL)
     { /* problem reading database index */
	if (ferror(db_pointer)) /* was there an error reading? */
	{
	   db_errorno=errno;
           break; /* get out of loop */
	}
	else /* must have been an EOF */
	{
#ifdef DEBUG
 printf("next_solved() - at end of file\n");
#endif
	   rewind(db_pointer);
	   current=0;
           if (start==0) /* we started the current search at the top */
               break; /* of the file, so there are no solved problems */
           else /* there could be more */
	      continue; /* go to top of loop */
        } /* end else it's EOF */
     }
     else /* no problem reading index file */
     {
        current+=sizeof(index_record); /* increment the current file position */
#ifdef DEBUG
 printf("next_solved() - starting file location = %d\n", start);
 printf("next_solved() - current file location = %d\n", current);
 print_index(index);
#endif
     } /* end else no problem */
  
    if ((index->mrd==LINKED) && links) /* problem was linked to current leaf
					* and we are following links */
    {
#ifdef DEBUG
 printf("next_solved() - problem is linked\n");
#endif

       if (check_index(db_pointer, index)==NULL) /* update entry */
	  break; /* error updating index entry, so break out of loop */
    } /* end if LINKED problem */

/* if we've found a solved problem that hasn't been deleted, then we've got
 * what we came here for, so set the flag and lets go. */

    if ((index->sf==SOLVED) && ((index->mrd!=GONE) && (index->mrd!=MOVED)))
    {
#ifdef DEBUG
 printf("next_solved() - found a solved problem\n");
#endif
       if ((index->mrd!=LINKED) || ((index->mrd==LINKED) && links))
          no_solved=FALSE;
#ifdef DEBUG
 else
    printf("\tproblem is linked and we are not following links\n");
#endif
    } /* end if !GONE and !MOVED */
#ifdef DEBUG
 else
    printf("next_solved() - problem is either unsolved or has been deleted\n");
#endif
  } /* end while */

  if (no_solved) /* we either rolled the index over once or there */
  {		 /* was an error */
     if (current==start)
     {
#ifdef DEBUG
 printf("next_solved() - no solved problems found\n");
#endif
        db_errorno=ROLLEDOVER;
     }
     return(NULL);
  } /* end if no_solved */
  start=current;
  return(index);
} /* end next_solved */


index_record *next_unsolved(FILE *db_pointer, int new, int links, 
			    index_record *index)
{
  static long start;
  long current;
  int no_unsolved;

#ifdef DEBUG
 printf("\nIn next_unsolved() - searching ");
 if (new)
    printf("NEW leaf\n");
 else
    printf("PREVIOUS leaf\n");
#endif

  if (new) /* searching a new database leaf */
     current=start=0;
  else /* searching old leaf so seek to last position */
  {
     fseek(db_pointer, start, SEEK_SET);
     current=start;
  }
  no_unsolved=TRUE;

  while (no_unsolved)
  {
     if (fread(index, sizeof(index_record), 1, db_pointer)<1)
     { /* problem reading database index */
	if (ferror(db_pointer)) /* was there an error reading? */
	{
	   db_errorno=errno;
           break; /* get out of loop */
	}
	else /* must have been an EOF */
	{
#ifdef DEBUG
 printf("next_unsolved() - at end of file\n");
#endif
	   rewind(db_pointer);
	   current=0;
           if (start==0) /* we started the current search at the top */
               break; /* of the file, so there are no unsolved problems */
           else /* there could be more */
	      continue; /* go to top of loop */
        } /* end else it's EOF */
     }
     else /* no problem reading index file */
     {
        current+=sizeof(index_record); /* increment the current file position */
#ifdef DEBUG
 printf("next_unsolved() - starting file location = %d\n", start);
 printf("next_unsolved() - current file location = %d\n", current);
 print_index(index);
#endif
     } /* end else no problem */
  
    if ((index->mrd==LINKED)  && links)/* problem was linked to current leaf
					* and we're following links */
    {
#ifdef DEBUG
 printf("next_unsolved() - problem is linked\n");
#endif

       if (check_index(db_pointer, index)==NULL) /* update entry */
	  break; /* error updating index entry, so break out of loop */
    } /* end if LINKED problem */

/* if we've found an unsolved problem that hasn't been deleted, then we've got
 * what we came here for, so set the flag and lets go. */

    if ((index->sf==UNSOLVED) && ((index->mrd!=GONE) && (index->mrd!=MOVED)))
    {
#ifdef DEBUG
 printf("next_unsolved() - found an unsolved problem\n");
#endif
       if ((index->mrd!=LINKED) || ((index->mrd==LINKED) && links))
          no_unsolved=FALSE;
#ifdef DEBUG
 else
    printf("\tproblem is linked and we are not following links\n");
#endif
    } /* end if !GONE and !MOVED */
#ifdef DEBUG
 else
    printf("next_unsolved() - problem has been either solved or deleted\n");
#endif

  } /* end while */

  if (no_unsolved) /* we either rolled the index over once, or there */
  {		   /* was an error */
     if (current==start)
     {
#ifdef DEBUG
 printf("next_unsolved() - no unsolved problems\n");
#endif
        db_errorno=ROLLEDOVER;
     }
     return(NULL);
  }
  start=current;
  return(index);

} /* end next_unsolved */


index_record *find_problem(FILE *db_pointer, char *prid, index_record *index)
{

#ifdef DEBUG
  printf("\nIn find_problem()\n");
  printf("\tPRID of problem to find = %s\n", prid);
#endif

  if (search_index(db_pointer, prid, index, 0L)==NULL)
     return(NULL); /* error locating problem desired */

#ifdef DEBUG
  printf("find_problem() - Found index entry of desired problem\n");
#endif

  if (index->mrd==LINKED) /* problem was found and it was linked to this */
  {	/* leaf, so update the entry */

#ifdef DEBUG
 printf("find_problem() - problem given is linked\n");
#endif

     if (check_index(db_pointer, index)==NULL)
	return(NULL); /* problem updating index entry */
  }
  if (index->mrd==GONE)
  {
#ifdef DEBUG
 printf("find_problem() - Problem has been deleted\n");
#endif
     db_errorno=DELETED;
     return(NULL);
  }

/* else everything worked so return the index record full of info. */
  return(index);
} /* end find_problem */


index_record *search_index(FILE *db_pointer, char *prid, index_record *index,
				long offset)
{
  long bottom,     /* of current search space */
       top,        /* of current search space */
       location;   /* current location in search space */
  int diff,
      not_found;

#ifdef DEBUG
  printf("\nIn search_index()\n");
  printf("\tprid - %s\n", prid);
#endif

  top=0;
  location=0;
  bottom=file_size(db_pointer, sizeof(index_record)); /* get the # of records 
					* in the index file to be searched */
  not_found=TRUE;

  while (not_found)
  {     /* get the next index record */
     if (top>bottom) /* we've gone past the end of the file */
     {
	db_errorno=NOTFOUND;
	break; /* out of loop */
     }

	 /* we're still in the file */
     location=((top+bottom)/2); /* find middle of search space */
/* go to the desired record */
     fseek(db_pointer, (location*(sizeof(index_record)))+offset, SEEK_SET);

#ifdef DEBUG
  printf("search_index() - After Seeking, Before Reading\n");
  printf("\tFile position = %d\n", ftell(db_pointer));
  printf("\tLocation variable = %d\n", location);
#endif

     if (fread(index, sizeof(index_record), 1, db_pointer)==NULL)
     {  /* there was a problem getting the record */
	if (ferror(db_pointer)) /* error reading file */
	   db_errorno=errno;
        else /* the end of the file was found before the problem */
	   db_errorno=NOTFOUND;
        break; /* out of loop */
     } /* end if */

#ifdef DEBUG
  printf("search_index() - searching index given\n");
  printf("\tPRID to find %s\n", prid);
  printf("\tCurrent PRID %s\n", index->prid);
#endif

     /* valid record was found */
     diff=strcmp(prid, index->prid); /* compare the ids */

     if (diff<0) /* index->prid < prid */
	bottom=location-1; /* search upper half of search space */
     else
	if (diff>0) /* index->prid > prid */
	   top=location+1; /* search lower half of search space */
        else /* we've found it */
	   not_found=FALSE;
  } /* end while */

/* we're out of the loop, if the search was sucessfull return the index entry
 * found, otherwise return NULL to signal an error.
 */
 
 if (not_found)
    return(NULL);
 else
    return(index);
} /* end search_index */


problem_record *create_record(char *db_path, index_record *index,
			      problem_record *pro_rec)
{
  reporter_record *rep_rec,	/* for ease of reading */
		  *rep_old;
  char *log;

  char path[DBPATHLEN],
       prid[MAXPRID],
       assfiles[FNAMELEN];
  index_record new_index;
  long size;
  int i;
  FILE *temp;

#ifdef DEBUG
 printf("\nIn create_record()\n");
#endif

  if (index->mrd==LINKED) /* problem record does not reside in this leaf */
  { /* so get the original index entry, index->ln_path is the db_path */
#ifdef DEBUG
 printf("create_record() - the problem given is a link\n");
#endif

     if ((temp=open_file(index->ln_path, ".index", "r", INDEX))==NULL)
     { /* error opening index file */
	return(NULL);
     }
     else /* index was opened, so get original index entry, ln_prid is the ID */
        if (find_problem(temp, index->ln_prid, &new_index)==NULL)
        {  /* problem finding index entry */
   	   close_file(temp);
	   return(NULL);
        }
        else /* we found the index entry */
        {
#ifdef DEBUG
 printf("create_record() - found original problem\n");
#endif
           close_file(temp);
           nl_strncpy(prid, new_index.prid, MAXPRID);/* prid id used to access 
						   * records */
           nl_strncpy(pro_rec->short_description, index->sd, MAXSUMLEN);
           nl_strncpy(path, index->ln_path, DBPATHLEN); /* path is used to access 
						      * records */
        } /* end found index entry */
  } /* end linked problem */
  else /* problem is not linked, so it resides in this leaf */
  {
     nl_strncpy(prid, index->prid, MAXPRID);
     nl_strncpy(pro_rec->short_description, index->sd, MAXSUMLEN);
     nl_strncpy(path, db_path, DBPATHLEN);
  } /* end else */

#ifdef DEBUG
 printf("create_record() - creating problem record\n");
#endif

  nl_strncpy(pro_rec->prid, index->prid, MAXPRID);/*user is unaware of any links */
  pro_rec->status=index->sf; /* set the status of the problem being created
			      * SOLVED or UNSOLVED */
  pro_rec->reop_rec=NULL; /* this will point to a reopened record, if the 
			   * problem has been reopened, will be set in 
			   * view_reopen() */

#ifdef DEBUG
 printf("create_record() - assigned prid and status\n");
 printf("\tprid - %s\n\tstatus - %d\n", pro_rec->prid, pro_rec->status);
#endif

  if (exists(PTSROOT, path, prid, ".rep")) /* get the reporters list, if 
					    * it exists */
  {
#ifdef DEBUG
 printf("create_record() - getting reporter records\n");
#endif

     sprintf(assfiles, "/%s.rep", prid);
     if ((temp=open_file(path, assfiles, "r", REP))==NULL)
     { /* problem opening reporters file */
        return(NULL);
     }
 /* opened reporters files */
 /* get the number of reporter records in the file */
     size=file_size(temp, sizeof(reporter_record));
     pro_rec->num_rep=size;
     if (pro_rec->reporter_list!=NULL)
	free(pro_rec->reporter_list); /*free any allocated space for the 
				       * last reporters list*/
     if ((rep_rec=(reporter_record *)calloc(size, sizeof(reporter_record)))
					==NULL)
     { /* error allocating space for the reporters list */
        db_errorno=errno;
	close_file(temp);
        return(NULL);
     }
     for (i=0; i<size; i++)
     {  /* get the reporters list */
        if (fread(&(rep_rec[i]), sizeof(reporter_record), 1, temp)==NULL)
        { /* error reading reporters file */
   	   db_errorno=errno;
	   close_file(temp);
	   return(NULL);
        }
     } /* end for */
     close_file(temp);
     pro_rec->reporter_list=rep_rec;
  } /* end if reporters file exists */
  else /* reporters file does not exist */
  {
#ifdef DEBUG
 printf("create_record() - no reporters file\n");
#endif
     if (index->sf==UNSOLVED) /* unsolved problems must have a reporters */
     { 			      /* list */
        db_errorno=NOREP;
        return(NULL);
     }
     pro_rec->num_rep=0;
     pro_rec->reporter_list=NULL;
  }
  /* now get the old reporters list, if it exists */

  if (exists(PTSROOT, path, prid, ".rep_old"))
  {  /* the file exists */
#ifdef DEBUG
 printf("create_record() - creating old reporters list\n");
#endif

     sprintf(assfiles, "/%s.rep_old", prid);
     if ((temp=open_file(path, assfiles, "r", REP))==NULL)
     { /* error opening old reporters file */
	db_errorno=errno;
        return(NULL);
     }
     size=file_size(temp, sizeof(reporter_record));
     pro_rec->num_old_rep=size;
     if (pro_rec->old_reporter_list!=NULL) /* free any space allocated for the
					    * last old rep. list */
	free(pro_rec->old_reporter_list);
     if ((rep_old=(reporter_record *)calloc(size, sizeof(reporter_record)))
	  ==NULL)
     {  /* error alloating space for the old reporters list */
	db_errorno=errno;
	close_file(temp);
	return(NULL);
     }
     for (i=0; i<size; i++)
     {  /* get the old reporters list */
	if (fread(&(rep_old[i]), sizeof(reporter_record),
		 1, temp)==NULL)
	{  /* error reading file */
	   close_file(temp);
	   db_errorno=errno;
	   return(NULL);
        }
     } /* end for, got old reporters list */
     close_file(temp);
     pro_rec->old_reporter_list=rep_old;
  } /* end getting old reporters list */
  else /* old reporters list doesn't exist */
  {
#ifdef DEBUG
 printf("create_record() - no old reporter records\n");
#endif
     if (index->sf==SOLVED)/* solved problems must have an old reporters list */     {
	db_errorno=NOREP;
	return(NULL);
     }
     pro_rec->num_old_rep=0;
     pro_rec->old_reporter_list=NULL;
  }
  /* we've got all the lists and such now for the log file */

#ifdef DEBUG
 printf("create_record() - getting problem log\n");
#endif

  sprintf(assfiles, "/%s.log", prid); /* the name of the problem log */
  if ((temp=open_file(path, assfiles, "r", REP))==NULL)
  {	/* error opening log file */
     db_errorno=errno;
     return(NULL);
  }
/* we opened the problem log, now find out how much memory to allocate */
 size=file_size(temp, sizeof(char));
 if (pro_rec->log_file!=NULL)
    free(pro_rec->log_file);

/* since the terminating NULL isn't in the log file, we need to allocate space 
 * for it and then append it ourselves.
 */
 if ((log=(char *)malloc(size+1))==NULL)
 {	/* error allocating space for problem log */
    db_errorno=errno;
    close_file(temp);
    return(NULL);
 }
#ifdef DEBUG
 printf("create_record() - allocated space for problem log\n");
 printf("\treading problem log into memory\n");
#endif

/* we allocated space for it now read it into memory */
 if (fread(log, sizeof(char), size, temp)<size)
 {	/* error reading log */
    close_file(temp);
    db_errorno=errno;
    return(NULL);
 }
 log[size]='\0';
 pro_rec->log_file=log;

#ifdef DEBUG
 printf("create_record() - created problem record\n");
#endif

/* we've got everything now so return the problem record */
  close_file(temp);
  return(pro_rec);
} /* end create_record() */


problem_record *partial_record(char *path, index_record *index,
				problem_record *pro_rec)
{

#ifdef DEBUG
 printf("\nIn partial_record()\n");
 printf("\tcreating partial record\n");
#endif

 nl_strncpy(pro_rec->prid, index->prid, MAXPRID);
 pro_rec->status=index->sf;
 nl_strncpy(pro_rec->short_description, index->sd, MAXSUMLEN);

#ifdef DEBUG
 printf("partial_record() - copied prid, problem status and short description\n");
#endif

 pro_rec->num_rep=0;
 if (pro_rec->reporter_list!=NULL)
 {
    free(pro_rec->reporter_list);
    pro_rec->reporter_list=NULL;
 }

#ifdef DEBUG
 printf("partial_record() - cleared reporters list\n");
#endif

 pro_rec->num_old_rep=0;
 if (pro_rec->old_reporter_list!=NULL)
 {
    free(pro_rec->old_reporter_list);
    pro_rec->old_reporter_list=NULL;
 }
#ifdef DEBUG
 printf("partial_record - cleared old reporters list\n");
#endif

 if (pro_rec->log_file!=NULL);
 {
    free(pro_rec->log_file);
    pro_rec->log_file=NULL;
 }
#ifdef DEBUG
 printf("artial_record() - cleared problem log\n");
#endif

 pro_rec->reop_rec=NULL;

 return(pro_rec);
} /* end partial_record() */


index_record *check_index(FILE *db_pointer, index_record *index)
{
  index_record org_index;
  FILE *org_temp;
  int changed; /* the index entry was changed */

#ifdef DEBUG
  printf("\nIn check_index()\n");
#endif

  changed=FALSE; /* set to false each time through */

  if ((org_temp=open_file(index->ln_path, ".index", "r", INDEX))==NULL)
  {  /* error opening index file */
     return(NULL);
  }

#ifdef DEBUG
  printf("check_index() - Looking for original problem in:\n");
  printf("\tDB Path = %s\n", index->ln_path);
  printf("\tPRID = %s\n", index->ln_prid);
#endif

  /* find the original index entry */
  if (search_index(org_temp, index->ln_prid, &org_index, 0L)==NULL)
  {  /* problem finding the index entry */
     close_file(org_temp);
     return(NULL);
  }
  else /* we found the index entry */
     close_file(org_temp);

#ifdef DEBUG
  printf("check_index() - found original problem\n");
#endif

  if (org_index.mrd==GONE)
  {  /* original problem has been deleted */

#ifdef DEBUG
  printf("check_index() - Original problem has been deleted\n");
#endif

     index->mrd=GONE;
  }
  else /* the original problem still exists */
  {
     if (org_index.mrd==MOVED)
     {  /* original problem has been moved to another leaf */
	/* so we need to propagate the change back to the index entry */

#ifdef DEBUG
  printf("check_index() - original problem has been moved\n");
#endif

        nl_strncpy(index->ln_path, org_index.ln_path, DBPATHLEN);
        nl_strncpy(index->ln_prid, org_index.ln_prid, MAXPRID);
        close_file(org_temp); /* we don't need this anymore */

/* since the recursive call to check_index() will take care of updating the
 * index entry we can just return that return value (that is we don't have
 * to update the entry twice
 */
        return(check_index(db_pointer, index));
     } /* end org_index entry == MOVED */

/* else the problem resides were we think so update the index record passed in,
 * but only if there are any changes made.
 */

#ifdef DEBUG
  printf("chech_index() - updating index entry\n");
#endif

     if (index->sf!=org_index.sf)
     {
	changed=TRUE;
        index->sf=org_index.sf;
     }
     if (index->ef!=org_index.ef)
     {
	changed=TRUE;
	index->ef=org_index.ef;
     }
     if (strcmp(index->account_name, org_index.account_name)!=0)
     {
	changed=TRUE;
	nl_strncpy(index->account_name, org_index.account_name, MAXNAMELEN);
     }
     if (strcmp(index->node_name, org_index.node_name)!=0)
     {
	changed=TRUE;
	nl_strncpy(index->node_name, org_index.node_name, MAXHOSTNAMELEN);
     }
     if (index->pid!=org_index.pid)
     {
	changed=TRUE;
	index->pid=org_index.pid;
     }
  } /* end original link has not been moved */

  if (changed) /* the index entry was changed (updated) */
  {
/* now that we've updated the index entry we need to change the file
 * entry to reflect the updated entry.  Since the file pointer is pointing
 * to the beginning of the next index entry, we need to back the pointer
 * up one record to the beginning of the one we need to change.
 */

#ifdef DEBUG
  printf("check_index() - writting out new index entry\n");
#endif

     fseek(db_pointer, -(long)sizeof(index_record), SEEK_CUR);
     if (lock_file(db_pointer)) /* lock the index before updating */
     { /* lock succeeded, so write the record */
        if (fwrite(index, sizeof(index_record), 1, db_pointer)<1)
        {  /* problem writting the record */
      	   db_errorno=errno;
   	   unlock_file(db_pointer);
	   fseek(db_pointer, 0L, SEEK_CUR); /* since the file is opened for 
				* update there needs to be a seek or EOF 
				* between reads and writes, this takes care 
				* of it. */
 	   return(NULL);
        }
        else /* the write succeeded */
        {
   	   unlock_file(db_pointer);
           fseek(db_pointer, 0L, SEEK_CUR); /* since the file is opened for 
				* update there needs to be a seek or EOF 
				* between reads and writes, this takes care 
				* of it. */
	   return(index);
        } /* end else write suceeded */
     } /* end lock file */
     else /* lock failed */
        return(NULL);
  } /* end index entry was changed (updated) */
} /* end check_index */


#ifdef DEBUG /* we only need these functions while debugging the database */

void print_index(index_record *index)
{
 printf("PRID - %s\tMRD - %d\tSF - %d\nEF - %d\n", index->prid,
		index->mrd, index->sf, index->ef);
 printf("Account Name - %s\nNode Name - %s\nPID - %d\n",
		index->account_name, index->node_name, index->pid);
 printf("LN PRID - %s\nLN PATH - %s\nShort Desc. - %s\nFR - %s\n\n", 
		index->ln_prid, index->ln_path, index->sd, index->fr);
} /* end print_index() */

void print_prob(problem_record *pro_rec)
{
 int i;

 printf("PRID - %s\nStatus - %d\nShort Desc. - %s\nReopened Record - %d\n", 
		pro_rec->prid, pro_rec->status, pro_rec->short_description,
		pro_rec->reop_rec);

 if (pro_rec->num_rep==0)
    printf("No Reporters.\n\n");
 else
    for (i=0; i<pro_rec->num_rep; i++)
    {
       printf("Reporter %d:\n\tAccount Name - %s\n\t", i,
                pro_rec->reporter_list[i].account_name);
       printf("Node Name - %s\n\tTime - %s\n\n",
                pro_rec->reporter_list[i].node_name,
                pro_rec->reporter_list[i].date_time);
    }

 if (pro_rec->num_old_rep==0)
    printf("No Old Reporters.\n\n");
 else
    for (i=0; i<pro_rec->num_old_rep; i++)
    {
       printf("Old Reporter %d:\n\tAccount Name - %s\n\t", i,
                pro_rec->old_reporter_list[i].account_name);
       printf("Node Name - %s\n\tTime - %s\n\n",
                pro_rec->old_reporter_list[i].node_name,
                pro_rec->old_reporter_list[i].date_time);
    }
 if (pro_rec->log_file==NULL)
    printf("No Log File\n");
 else
    printf("Log File:\n\n%s", pro_rec->log_file);
} /* end print_prob */

#endif
/* end of get_problem.c */
