/*
 * move_problem.c --
 * Chris Hyde
 * created: 
 */

/*
 * Copyright (c) 1992 Christopher G. Hyde
 * Copyright (c) 1992 University of Idaho, Moscow, Idaho.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation free of charge for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the University of Idaho
 * not be used in advertising or publicity pertaining to distribution of
 * the software without specific, written prior permission.  The University
 * of Idaho makes no representations about the suitability of this software
 * for any purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE UNIVERSITY OF IDAHO DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE UNIVERSITY OF IDAHO BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "zdbm.h"

int move_problem(char *old_db_path, char *new_db_path, char *prid)
{
 index_record index;
 char *tprid;
 FILE *temp;

#ifdef DEBUG
 printf("\nIn move_problem()\n");
 printf("\tOld DB path - %s\n\tNew DB path - %s\n", old_db_path,
		 new_db_path);
 printf("\tPRID - %s\n", prid);
#endif

 db_errorno=0;

 if (strcmp(old_db_path, new_db_path)==0)
 {	/* can't move a problem to the same leaf */
    db_errorno=SAMELEAF;
    return(FAIL);
 }

 if ((temp=open_file(old_db_path, ".index", "r+", INDEX))==NULL)
    return(FAIL);

 if (find_problem(temp, prid, &index)==NULL)
 {	/* error tring to find the problem to move */
    close_file(temp);
    return(FAIL);
 }

 if (index.ef==TRUE) /* the problem is currently being edited */
 {
    db_errorno=ETXTBSY; /* the problem is locked as far as the user is
			 * concerned */
    close_file(temp);
    return(FAIL);
 }

 if (index.mrd==GONE)
 {	/* can't move a deleted problem */
    db_errorno=DELETED;
    close_file(temp);
    return(FAIL);
 }

 if (index.mrd==LINKED)
    if (strcmp(new_db_path, index.ln_path)==0)
    {	/* can't move problem to the leaf that holds original link */
       db_errorno=SAMELEAF;
       close_file(temp);
       return(FAIL);
    }


#ifdef DEBUG
 printf("move_problem() - Found problem to move, now begin moving it\n");
#endif

 if (lock_file(temp)==FAIL)
 {	/* the lock failed */
    close_file(temp);
    return(FAIL);
 }

/* now physically move the problem requested */
 if ((tprid=move_record(old_db_path, new_db_path, &index))==NULL)
 {	/* the move failed */
    unlock_file(temp);
    close_file(temp);
    return(FAIL);
 }

#ifdef DEBUG
 printf("move_problem() - The problem has been moved, now mark original problem\n\t as deleted\n");
#endif

/* now move back to the begining of the index entry that was moved */
 fseek(temp, -(long)(sizeof(index_record)), SEEK_CUR);

 if ((index.mrd==ORGLINK) || (index.mrd==REOPENED)) 
 { /* the problem to move is either an original link or has been requested
    * for reopening so set up index entry so the move will travel down the 
    * links */
    index.mrd=MOVED;
    nl_strncpy(index.ln_path, new_db_path, DBPATHLEN);
    nl_strncpy(index.ln_prid, tprid, MAXPRID);
 }
 else /* the problem either resides here or is just a plain link */
    index.mrd=GONE; /* so set the index entry to reflect the move */

/* now write out the updated index entry */
 if (fwrite(&index, sizeof(index_record), 1, temp)<1)
 {	/* the write failed */
    db_errorno=errno;
    unlock_file(temp);
    close_file(temp);
    return(FAIL);
 }
 unlock_file(temp);
 close_file(temp);
 return(SUCESS);
} /* end move_problem */


char *move_record(char *old_db_path, char *new_db_path, index_record *index)
{
 char tprid[MAXPRID];
 char *tptr;
 int tmrd;
 long size;
 FILE *temp;

#ifdef DEBUG
 printf("\nIn move_record()\n");
 printf("\tOld DB path - %s\n\tNew DB path - %s\n", old_db_path,
		new_db_path);
#endif

 tmrd=index->mrd; /* store the old value of this field for replacement */
 nl_strncpy(tprid, index->prid, MAXPRID);

/* we need to open the index for write rather than append, since we overwrite
 * the index entry when finished.
 */
 if ((temp=open_file(new_db_path, ".index", "w", INDEX))==NULL)
    if (errno==ENOENT) /* the index does not exist */
       if (make_branch(new_db_path)==FAIL) /* so create it */
          return(NULL); /* there was a problem creating the new index */
       else /* we  created the index, so open it for writting */
          if ((temp=open_file(new_db_path, ".index", "w", INDEX))==NULL)
              return(NULL);
          else; /* dummy else to go with if above */
    else /* we couldn't open the file for some other reason, so quit */
       return(NULL);

/* we want to append to the new index, so seek to it. */
 fseek(temp, 0L, SEEK_END);

 if (lock_file(temp)==FAIL)/* lock the new index before making the new record */ {	/* the lock failed */
    close_file(temp);
    return(NULL);
 }
 if (index->mrd==ORGLINK) /* check to see if a link already exists in the
			  * new leaf */
    if (double_problem(temp, old_db_path, index->prid))
    { /* the problem already exists in the new leaf */
       db_errorno=SAMELEAF;
       close_file(temp);
       return(FAIL);
    }

 if (index->mrd==LINKED) /* check to see if a link already exists in the
			 * new leaf */
    if (double_problem(temp, index->ln_path, index->ln_prid))
    { /* the problem already exists in the new leaf */
       db_errorno=SAMELEAF;
       close_file(temp);
       return(FAIL);
    }

/* get the size of the index file, in records */
 if ((size=file_size(temp, sizeof(index_record)))==ERROR)
 {	/* there was an error getting the file size */
    unlock_file(temp);
    close_file(temp);
    return(NULL);
 }

/* now create a new problem id and stuff it in the index entry */
 if ((tptr=new_prid(size))==NULL)
 {	/* an error occured getting the new prid */
    unlock_file(temp);
    close_file(temp);
    return(NULL);
 }
 nl_strncpy(index->prid, tptr, MAXPRID);
 index->mrd=GONE; /* will keep the new entry from being read if the move
 		      * fails */


/* now write out the new index entry */
 if (fwrite(index, sizeof(index_record), 1, temp)<1)
 {	/* the write failed, so restore the index entry before returning */
    db_errorno=errno;
    index->mrd=tmrd;
    nl_strncpy(index->prid, tprid, MAXPRID);
    unlock_file(temp);
    close_file(temp);
    return(NULL);
 }
 unlock_file(temp); /* we don't need to keep the index locked */

#ifdef DEBUG
 printf("move_record() - Created new index entry, and wrote it to the new index\n");
 printf("\tNow move over the associated files\n");
#endif

/* now move the files over to the new leaf */
 if (exists(PTSROOT, old_db_path, tprid, ".rep")) /* if the reporters file
					           * exists then copy it over */
    if (copy_file(old_db_path, tprid, ".rep", new_db_path, tptr, ".rep")==FAIL)
    {	/* the copy failed, so restore the index entry before returning */
       index->mrd=tmrd;
       nl_strncpy(index->prid, tprid, MAXPRID);
       close_file(temp);
       return(NULL);
    }

 if (exists(PTSROOT, old_db_path, tprid, ".rep_old")) /* if the rep_old file
					      * exists, copy it over to the 
					      * new leaf */
    if (copy_file(old_db_path, tprid, ".rep_old",
		   new_db_path, tptr, ".rep_old")==FAIL)
    {	/* the copy failed, so restore the index entry before returning */
       index->mrd=tmrd;
       nl_strncpy(index->prid, tprid, MAXPRID);
       close_file(temp);
       remove_file(new_db_path, tptr, ".rep");
       return(NULL);
    }

 if (exists(PTSROOT, old_db_path, tprid, ".log")) /* if the log file exists,
 					       * copy it over to the new leaf */
    if (copy_file(old_db_path, tprid, ".log", 
   		  new_db_path, tptr, ".log")==FAIL)
    {	/* the copy failed, so restore the index entry before returning */
       index->mrd=tmrd;
       nl_strncpy(index->prid, tprid, MAXPRID);
       close_file(temp);
       remove_file(new_db_path, tptr, ".rep");
       remove_file(new_db_path, tptr, ".rep_old");
       return(NULL);
    }

#ifdef DEBUG
 printf("move_record() - The files were moved sucessfully, now delete the originals\n");
#endif

/* the file moves succeeded so mark the new index entry as before and overwrite
 * the old one, remove the old files and return the old index entry to its
 * original values 
 */
 fseek(temp, -(long)(sizeof(index_record)), SEEK_CUR);
 index->mrd=tmrd;
 if (fwrite(index, sizeof(index_record), 1, temp)<1)
 {   /* the write failed, so restore index entry and remove the cpoied files */
    db_errorno=errno;
    close_file(temp);
    nl_strncpy(index->prid, tprid, MAXPRID);
    remove_file(new_db_path, tptr, ".rep");
    remove_file(new_db_path, tptr, ".rep_old");
    remove_file(new_db_path, tptr, ".log");
    return(NULL);
 }
/* else everything worked so restore the index entry and remove the original
 * files
 */
 close_file(temp);
 nl_strncpy(index->prid, tprid, MAXPRID);
 remove_file(old_db_path, tprid, ".rep");
 remove_file(old_db_path, tprid, ".rep_old");
 remove_file(old_db_path, tprid, ".log");
 return(tptr);
} /* end move_record */


int link_problem(char *old_db_path, char *new_db_path, char *prid)
{
 index_record index,
	      new_index;
 long size;
 FILE *temp;
 char *tptr;

#ifdef DEBUG
  printf("\nIn link_problem()\n");
  printf("\told_db_path - %s\n", old_db_path);
  printf("\tnew_db_path - %s\n\tprid - %s\n", new_db_path, prid);
#endif

 db_errorno=0;

 if (strcmp(old_db_path, new_db_path)==0)
 {	/* can't link a problem to the same leaf */
    db_errorno=SAMELEAF;
    return(FAIL);
 }

 if ((temp=open_file(old_db_path, ".index", "r+", INDEX))==NULL)
    return(FAIL);

 if (find_problem(temp, prid, &index)==NULL)
 {	/* could not find problem requested */
     close_file(temp);
     return(FAIL);
 }

#ifdef DEBUG
  printf("link_problem() - Found problem desired\n");
  printf("\tChecking problem found.\n");
#endif

 if (index.mrd==GONE) /* can't link a deleted problem */
 {
    db_errorno=DELETED;
    close_file(temp);
    return(FAIL);
 }

 if (index.mrd==LINKED)/* problem does not reside in leaf given so set up the */
 {	/* new index record using the original problem as the problem that
	 * will be linked to
 	 */
    if (strcmp(new_db_path, index.ln_path)==0)
    {	/* can't link to a leaf in which the problem already resides */
       db_errorno=SAMELEAF;
       close_file(temp);
       return(FAIL);
    }
#ifdef DEBUG
  printf("link_problem() - Problem is linked, and is OK for linking\n");
#endif
    nl_strncpy(new_index.ln_path, index.ln_path, DBPATHLEN);
    nl_strncpy(new_index.ln_prid, index.ln_prid, MAXPRID);
    close_file(temp);
 } /* end if LINKED */
 else /* problem resides in the leaf given, so set up the new index record */
 {    /* using the problem from this leaf. If the problem is already an original
       * link then don't change the original index entry, otherwise this is the
       * first link to this problem so mark it as the original link in a series
       * of links and update the index entry
       */

#ifdef DEBUG
  printf("link_problem() - Problem is not linked, and is OK for linking\n");
#endif

    if (index.mrd!=ORGLINK)
    {
       index.mrd=ORGLINK;
       fseek(temp, -(long)(sizeof(index_record)), SEEK_CUR);
       if (lock_file(temp)==FAIL)
       {	/* the lock failed */
          close_file(temp);
          return(FAIL);
       }
       if (fwrite(&index, sizeof(index_record), 1, temp)<1)
       {	/* error writing to file */
          db_errorno=errno;
          unlock_file(temp);
          close_file(temp);
          return(FAIL);
       }
    } /* end not ORGLINK */
/* else the update succeed so set up the new index record */
    unlock_file(temp);
    close_file(temp);
    nl_strncpy(new_index.ln_prid, old_db_path, DBPATHLEN);
    nl_strncpy(new_index.ln_prid, index.prid, MAXPRID);
 } /* end else problem resides in this leaf */

/* finish assigning values to the new index record */
 new_index.mrd=LINKED; /* mark new problem as liked */
 new_index.sf=index.sf;
 new_index.ef=index.ef;
 nl_strncpy(new_index.account_name, index.account_name, MAXNAMELEN);
 nl_strncpy(new_index.node_name, index.node_name, MAXHOSTNAMELEN);
 new_index.pid=index.pid;
 nl_strncpy(new_index.sd, index.sd, MAXSUMLEN);
 nl_strncpy(new_index.fr, index.fr, PADLEN);

#ifdef DEBUG
  printf("link_problem() - Adding new index entry\n");
#endif

/* add new index entry to the new index */
 if ((temp=open_file(new_db_path, ".index", "a", INDEX))==NULL)
    if (errno==ENOENT) /* the index does not exist */
       if (make_branch(new_db_path)==FAIL) /* so create it */
          return(NULL); /* there was a problem creating the new index */
       else /* we  created the index, so open it for update */
          if ((temp=open_file(new_db_path, ".index", "a", INDEX))==NULL)
              return(NULL);
          else; /* dummy else to go with if above */
    else /* we couldn't open the file for some other reason, so quit */
       return(NULL);

 if (lock_file(temp)==FAIL)
 {	/* the lock failed */
    close_file(temp);
    return(FAIL);
 }
 if (index.mrd==ORGLINK) /* check the new leaf to see if the problem is
			  * already in the new leaf */
    if (double_problem(temp, old_db_path, index.prid))
    { /* problem is already in new leaf */
       db_errorno=SAMELEAF;
       close_file(temp);
       return(FAIL);
    }
 if (index.mrd==LINKED) /* check the new leaf to see if the problem is
			 * already in the new leaf */
    if (double_problem(temp, index.ln_path, index.ln_prid))
    { /* problem is already in new leaf */
       db_errorno=SAMELEAF;
       close_file(temp);
       return(FAIL);
    }

 if ((size=file_size(temp, sizeof(index_record)))==ERROR)
 {	/* could not get file size */
    unlock_file(temp);
    close_file(temp);
    return(FAIL);
 }
 if ((tptr=new_prid(size))==NULL)
 {	/* error finding new prid */
    unlock_file(temp);
    close_file(temp);
    return(FAIL);
 }
 nl_strncpy(new_index.prid, tptr, MAXPRID);

 if (fwrite(&new_index, sizeof(index_record), 1, temp)<1)
 {	/* error writing the new record */
    db_errorno=errno;
    unlock_file(temp);
    close_file(temp);
    return(FAIL);
 }
/* else the problem was written OK so signal sucess */

#ifdef DEBUG
  printf("link_problem() - New prid of problem - %s\n", tptr);
#endif

 unlock_file(temp);
 close_file(temp);
 return(SUCESS);
} /* end link_problem */

/* end of move_problem.c */
