/*
 * update_problem.c --
 * Chris Hyde
 * created: 
 */

/*
 * Copyright (c) 1992 Christopher G. Hyde
 * Copyright (c) 1992 University of Idaho, Moscow, Idaho.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation free of charge for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the University of Idaho
 * not be used in advertising or publicity pertaining to distribution of
 * the software without specific, written prior permission.  The University
 * of Idaho makes no representations about the suitability of this software
 * for any purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE UNIVERSITY OF IDAHO DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE UNIVERSITY OF IDAHO BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "zdbm.h"


int solve_problem(char *db_path, char *prid)
{
 index_record index;
 char tprid[MAXPRID],
      tpath[DBPATHLEN];
 FILE *temp;

#ifdef DEBUG
 printf("\nIn solve_problem()\n");
 printf("\tSolving %s/%s\n", db_path, prid);
#endif

 if ((temp=open_file(db_path, ".index", "r+", INDEX))==NULL)
    return(FAIL);

 if (find_problem(temp, prid, &index)==NULL)
 {	/* error locating desired problem */
    close_file(temp);
    return(FAIL);
 }

#ifdef DEBUG
 printf("solve_problem() - found problem to solve\n");
#endif

 if (index.mrd==GONE)
 {	/* you can't solve a deleted problem */
    db_errorno=DELETED;
    close_file(temp);
    return(FAIL);
 }
    
 if (index.sf==SOLVED)
 {	/* problem has already been solved */
    close_file(temp);
    return(SUCESS);
 }

 if (index.mrd==LINKED) /* problem does not reside in this leaf */
 {

#ifdef DEBUG
 printf("solve_problem() - problem given is linked\n");
#endif

/* since find_problem overwrites the data in index, we need to store the
 * path and prid of the original problem somewhere else.
 */
    nl_strncpy(tprid, index.ln_prid, MAXPRID);
    nl_strncpy(tpath, index.ln_path, DBPATHLEN);
    close_file(temp); /* we don't need this index anymore */

    if ((temp=open_file(tpath, ".index", "r+", INDEX))==NULL)
    {
       return(FAIL);
    }

    if (find_problem(temp, tprid, &index)==NULL)
    {	/* error locating original problem */
       close_file(temp);
       return(FAIL);
    }

#ifdef DEBUG
 printf("solve_problem() - found the original link\n");
#endif
    if (solve_record(temp, &index, tpath, tprid)==FAIL)
    {
       close_file(temp);
       return(FAIL);
    }
    else
    {
       close_file(temp);
       return(SUCESS);
    } /* end else */
 } /* end LINKED problem */
 else
 { /* the problem is not linked */
    if (solve_record(temp, &index, db_path, prid)==FAIL)
    { /* problem solving problem */
       close_file(temp);
       return(FAIL);
    }
    else
    {
       close_file(temp);
       return(SUCESS);
    }
 } /* end else not LINKED */
} /* end solve_problem */


int solve_record(FILE *fp, index_record *index, char *path, char *prid)
{
 int test;

#ifdef DEBUG
 printf("In solve_record()\n");
 printf("\trecord to solve - %s/%s\n", path, prid);
#endif

 if (lock_file(fp)==FAIL)
    return(FAIL);

 index->sf=SOLVED;

/* since fp is pointing to the end of the record we want to update, we need
 * to move the pointer back one record
 */

 fseek(fp, -(long)sizeof(index_record), SEEK_CUR);
 if (fwrite(index, sizeof(index_record), 1, fp)<1)
 {
    db_errorno=errno;
    unlock_file(fp);
    return(FAIL);
 }

#ifdef DEBUG
 printf("solve_record() - wrote out new index entry\n");
#endif

 if (exists(PTSROOT, path, prid, ".rep_old"))
    test=append(path, prid, ".rep", prid, ".rep_old");
 else
    test=rename_file(path, prid, ".rep", path, prid, ".rep_old");

 unlock_file(fp);
 if (test==SUCESS)
    remove_file(path, prid, ".rep");

 return(test);
} /* end solve_record() */


int delete_problem(char *db_path, char *prid, int perm)
{
 index_record index;
 FILE *temp;

#ifdef DEBUG
 printf("\nIn delete_problem()\n");
 printf("\tPath to problem - %s\n", db_path);
 printf("\tProblem ID - %s\n", prid);
 printf("\tDelete permission - %d\n", perm);
#endif

 db_errorno=0;

 if ((temp=open_file(db_path, ".index", "r+", INDEX))==NULL)
    return(FAIL);

 if (find_problem(temp, prid, &index)==NULL)
 {
    close_file(temp);
    return(FAIL);
 }

#ifdef DEBUG
 printf("delete_problem() - Found problem to delete\n");
#endif

/* if the problem to be deleted is an original link or the problem has been
 * requested for reopening, and no permission is given then return an error
 */
 if (((index.mrd==ORGLINK) || (index.mrd==REOPENED)) && (perm==DELETENOTOK)) 
 {
    db_errorno=LINKERR;
    close_file(temp);
    return(FAIL);
 }

/* if (index.sf==TRUE) */
 if (index.ef==TRUE)  /* Typo fixed by DC, 2/13/93 */
 { /* the problem is currently being edited */
    db_errorno=ETXTBSY;
    close_file(temp);
    return(FAIL);
 }

/* reset to the begining of the current index entry, the one to be deleted */
 fseek(temp, -(long)(sizeof(index_record)), SEEK_CUR);
 index.mrd=GONE;

/* lock the index file before updating the index entry */
 if (lock_file(temp)==FAIL)
 {
    close_file(temp);
    return(FAIL);
 }

/* write out the new index entry */
 if (fwrite(&index, sizeof(index_record), 1, temp)<1)
 {	/* the write failed */
    db_errorno=errno;
    unlock_file(temp);
    close_file(temp);
    return(FAIL);
 }
 else /* the write succeeded */
 {
    unlock_file(temp);
    close_file(temp);
 }

#ifdef DEBUG
 printf("delete_problem() - Marked index entry of problem as deleted\n");
 printf("delete_problem() - Now removing associated files\n");
#endif

/* after updating the index, remove all the associated files, if they exist */
 remove_file(db_path, prid, ".rep");
 remove_file(db_path, prid, ".rep_old");
 remove_file(db_path, prid, ".log");
 return(SUCESS);
} /* end delete_problem */


char *add_problem(reporter_record *rep_rec, char *problem_description,
		  char *short_description, char *db_path)
{
 index_record index;
 FILE *temp,
      *temp2;
 long size;
 char *tptr;

#ifdef DEBUG
 printf("\nIn add_problem()\n");
 printf("\treporter_record.account_name = %s\n",
			 rep_rec->account_name);
 printf("\treporter_record.node_name = %s\n", rep_rec->node_name);
 printf("\treporter_record.date_time = %s\n", rep_rec->date_time);
 printf("\tproblem_description = %s\n", problem_description);
 printf("\tshort_description = %s\n", short_description);
 printf("\tdb_path = %s\n", db_path);
#endif

 db_errorno=0;

/* set up the new index record */
 index.mrd=GONE;
 index.sf=UNSOLVED;
 index.ef=FALSE;
 index.account_name[0]='\0';
 index.node_name[0]='\0';
 index.pid=0;
 nl_strncpy(index.sd, short_description, MAXSUMLEN);
 index.ln_prid[0]='\0';
 index.ln_path[0]='\0';
 index.fr[0]='\0';

#ifdef DEBUG
 printf("add_problem() - created new index record\n");
#endif

 if ((temp=open_file(db_path, ".index", "w+", INDEX))==NULL)
    if (errno==ENOENT) /* the index does not exist */
       if (make_branch(db_path)==FAIL) /* so create it */
          return(NULL); /* there was a problem creating the new index */
       else /* we  created the index, so open it for update */
          if ((temp=open_file(db_path, ".index", "w+", INDEX))==NULL)
              return(NULL);
          else; /* dummy else to go with if above */
    else /* we couldn't open the file for some other reason, so quit */
       return(NULL);

 if (lock_file(temp)==FAIL)
 {	/* the lock failed */
    close_file(temp);
    return(NULL);
 }
 if ((size=file_size(temp, sizeof(index_record)))==ERROR)
 {	/* could not get file size */
    unlock_file(temp);
    close_file(temp);
    return(NULL);
 }
 if ((tptr=new_prid(size))==NULL)
 {	/* error getting new id */
    unlock_file(temp);
    close_file(temp);
    return(NULL);
 }

#ifdef DEBUG
 printf("add_problem() - got new problem id - %s\n", tptr);
#endif

 nl_strncpy(index.prid, tptr, MAXPRID);

/* now that the record has been created, move to the end of the index file
 * and write it */
 fseek(temp, 0L, SEEK_END);
 if (fwrite(&index, sizeof(index_record), 1, temp)<1)
 {	/* the write failed */
    db_errorno=errno;
    unlock_file(temp);
    close_file(temp);
    return(NULL);
 }

#ifdef DEBUG
 printf("add_problem() - wrote out new index record\n");
#endif

/* else the write suceeded, so unlock the index and create the problem files */
 unlock_file(temp);

	/* create the reporter file */
 if ((temp2=create_file(db_path, index.prid, ".rep",
			FILE_PERMS))==NULL)
 {	/* the creation failed */
    close_file(temp);
    return(NULL);
 }

#ifdef DEBUG
 printf("add_problem() - created reporter file for new problem\n");
#endif

 if (fwrite(rep_rec, sizeof(reporter_record), 1, temp2)<1)
 {	/* the write failed */
    db_errorno=errno;
    close_file(temp);
    close_file(temp2);
    remove_file(db_path, index.prid, ".rep");
    return(NULL);
 }
 close_file(temp2);

#ifdef DEBUG
 printf("add_problem() - wrote out reporter record\n");
#endif

/* making the reporter file suceeded, so create the log file */
if (write_log(db_path, tptr, problem_description)==FAIL)
{
   close_file(temp);
   remove_file(db_path, tptr, ".rep");
   return(FAIL);
}

#ifdef DEBUG
 printf("add_problem() - created log file for new problem\n");
#endif

/* the two problem files were created, so go back and mark the index entry 
 * as not deleted, so it will be visable to the user.
 */
 fseek(temp, -(long)sizeof(index_record), SEEK_CUR);
 index.mrd=NULL;
 if (fwrite(&index, sizeof(index_record), 1, temp)<1)
 {	/* the write failed */
    db_errorno=errno;
    close_file(temp);
    remove_file(db_path, index.prid, ".rep");
    remove_file(db_path, index.prid, ".log");
    return(NULL);
 }

#ifdef DEBUG
 printf("add_problem() - updated index entry\n");
#endif

/* else the whole thing suceeded, so return the new prid */
 close_file(temp);
 return(index.prid);
} /* end add_problem */


char *add_reporter(char *db_path, char *prid, reporter_record *rep_rec)
{
 index_record index;
 char *tprid,
      *path;
 FILE *temp;

#ifdef DEBUG
 printf("\nIn add_reporter()\n");
 printf("\tPath = %s.\n\tPrid = %s.\n", db_path, prid);
#endif

 db_errorno=0;

 if ((temp=open_file(db_path, ".index", "r+", INDEX))==NULL)
    return(NULL);
 if (find_problem(temp, prid, &index)==NULL)
 {	/* could not find problem requested */
    close_file(temp);
    return(NULL);
 }
 close_file(temp); /* we found what we needed */

#ifdef DEBUG
 printf("add_reporter() - found index record\n");
#endif

/* now check to be sure a reporter can be added */
 if (index.mrd==GONE)
 {	/* problem was deleted, so you can't add a reporter */
    db_errorno=DELETED;
    return(NULL);
 }
 if (index.sf==SOLVED)
 {	/* can't add a reporter to a solved problem */
    db_errorno=ISSOLVED;
    return(NULL);
 }

 if (index.mrd==LINKED) /* problem does not reside in this leaf, so */
 {	/* update the entry */
    path=index.ln_path; /* path and tprid are used to access files */
    tprid=index.ln_prid;
 } /* end if LINKED problem */
 else /* the problem resides in this leaf, so assign values to path and tprid */
 {
    path=db_path;
    tprid=prid;
 } /* end else problem not LINKED */

/* we know where the reporter file is (path) and its name (tprid), so now 
 * append the reporter record to the end of the reporter file 
 */
 if (append_reporter(path, tprid, rep_rec)==FAIL)
    return(NULL); /* the appending failed */
 else
    return(prid); /* the appending succeed */
} /* end add_reporter */


int append_reporter(char *db_path, char *prid, reporter_record *rep_rec)
{
 static char fn[FNAMELEN];
 FILE *temp;

#ifdef DEBUG
 printf("\nIn append_reporter()\n");
 printf("\tDB Path = %s\n", db_path);
 printf("\tPRID = %s\n", prid);
#endif

 sprintf(fn, "/%s.rep", prid);
 if ((temp=open_file(db_path, fn, "r+", REP))==NULL)
    return(FAIL);
 if (lock_file(temp)==FAIL)
 {	/* the lock failed */
    close_file(temp);
    return(FAIL);
 }
 if (double_reporter(temp, rep_rec))
 {	/* reporter is already listed */
    unlock_file(temp);
    close_file(temp);
    return(FAIL);
 }

/* no double reporter, so move to end of file and add the new reporter */
 fseek(temp, 0L, SEEK_END);
 if (fwrite(rep_rec, sizeof(reporter_record), 1, temp)<1)
 {	/* the write failed */
    db_errorno=errno;
    unlock_file(temp);
    close_file(temp);
    return(FAIL);
 }
/* else the write suceeded, so cleanup and signal sucess */
 unlock_file(temp);
 close_file(temp);
 return(SUCESS);
} /* end append_reporter */


int double_reporter(FILE *db_pointer, reporter_record *rep_rec)
{
 reporter_record in_rec;
 int found=FALSE;

#ifdef DEBUG
 printf("\nIn double_reporter()\n");
#endif

 while (found==FALSE)
 {
    if (fread(&in_rec, sizeof(reporter_record), 1, db_pointer)<1)
    {
       if (ferror(db_pointer))
       { /* there was an I/O error */
          db_errorno=errno;
          found=ERROR;
#ifdef DEBUG
 printf("double_reporter() - I/O error %d\n", errno);
#endif
       } /* end I/O error */
#ifdef DEBUG
       else
          printf("double_reporter() - no doubles\n");
#endif
       break;
    }

#ifdef DEBUG
  printf("double_reporter() - account name looking for '%s'\n", 
		rep_rec->account_name);
  printf("double_reporter() - current account name '%s'\n",
		in_rec.account_name);
#endif

    if (strcmp(in_rec.account_name, rep_rec->account_name)==0)
    {
       db_errorno=DOUBLEREP;
       found=TRUE;
#ifdef DEBUG
 printf("double_reporter() - found a double\n");
#endif
    }
 }
 return(found);
} /* end double_reporter */

/* end update_problem.c */
